#!/bin/sh 
# \
exec wish "$0" "$@"

;# NAME:   testHarness.tcl
;# AUTHOR: Clif Flynt
;# DATE:   Mar 5, 2002
;# DESC:   A test harness for testing web pages.
;#         Use tcpdumpTEXT.tcl to extract HTTP transfers from tcpdump
;#         output.
;#         Use getGET.tcl to convert the text output into a tcl script.
;#         Invoke the testHarness with the name of that output and a
;#         status file identifier.
;# 
;#         tclsh tcpdumpTEXT.tcl <tcpdumpOutput | tclsh getGET.tcl >test.tcl
;#         wish testHarness.tcl test.tcl testID
;#
;#  This code was described in login magazine at www.usenix.org
;# 
;# PARAMETERS:  body.tcl statusLogFileIdentifier
;#
;# RCSID: $Header: /home/cvs/WWWTest/testHarness.tcl,v 1.1.1.1 2002/03/05 19:20:44 clif Exp $
;#
;# This code copyright 2002 to Clif Flynt
;# 9300 Fleming Rd
;# Dexter, MI  48130
;# clif@cflynt.com
;# All Rights Reserved.

package require http

set Test(logFile) "output_[pid]"

################################################################
# proc logData {string}--
#    Write data to the status log file
# Arguments
#   string:	A string to write to the file
# 
# Results
#   
# 
proc logData {string} {
    global Test

    .t insert end "$string\n"
    puts $Test(statusLog) $string
    flush $Test(statusLog)
}

################################################################
# proc comparePages {file1 file2}--
#    Compares two files: Converts obvious date references to a common
#    date to avoid false positives.
# Arguments
#   file1:	Name of test file
#   file2: 	Name of gold file
# 
# Results
#   Returns a single line yea/nea report
# 
proc comparePages {file1 file2} {
    set if1 [open $file1 r]
    set if2 [open $file2 r]

    set d1 [read $if1]
    set d2 [read $if2]

    close $if1
    close $if2

    set today [clock format [clock seconds] -format {%m/%d/%Y}]

    if {[string first $d1 $d2] != 0} {
	regsub -all {"[0-9][0-9]?/[0-9][0-9]?/[0-9][0-9]?[0-9]?[0-9]?"} \
	  $d2 "$today" d2

	regsub -all {"[0-9][0-9]?/[0-9][0-9]?/[0-9][0-9]?[0-9]?[0-9]?"} \
	  $d1 "$today" d1

	if {[string first $d1 $d2] != 0} {
	    set rply "FAILED MATCH IN $file1"
	} else {
	    set rply "Clean copy in $file1 Except date fields"
	}
    } else {
	set rply "Clean copy in $file1"
    }
    return $rply
}

################################################################
# proc checkResults {}--
#    Step through the files in the test directory and compare
#    them with the gold equivalent
# Arguments
#   NONE
# 
# Results
#   Adds lines of information to the GUI text window.
# 
proc checkResults {} {
    global Test 
    if {[file exists $Test(base)_gold]} {
	set failCount 0

	foreach f [glob $Test(logFile)*] {

	    foreach {goldBase id} [split $Test(base) "_"] { }
	    foreach {base run id} [split $f "_"] { }

	    set if2 [open ${goldBase}_gold/${base}_${id} r]
	    set rslt [comparePages $f ${goldBase}_gold/${base}_${id}]
	    if {[string first FAILED $rslt] >= 0} {
		incr failCount
	    }

	    .l configure -text $rslt
	    logData $rslt
	}
	set msg "Check completed: $failCount errors"
    } else {
	set msg "No check files found in $Test(base)_gold\nMaking the Gold Directory and filling it with this run!"
	file mkdir $Test(base)_gold
	foreach f [glob output output*[pid]*] {
	    regsub "_[pid]_" $f "_" newf
	    file copy $f $Test(base)_gold/$newf
	    file delete $f
	}
    }

    .l configure -text $msg
    logData $msg
}

################################################################
# proc done {url id startTime token}--
#    Invoked when an HTTP transfer is complete.
# Arguments
#   url 	The url that was accessed
#   id 		The ID number we assign for this transfer
#   startTime 	The start time of the transfer, in seconds
#   token	The token added by http::geturl
# 
# Results
#   Logs status, 
#   Saves output for later checking
#   Updates GUI
# 
proc done {url id startTime token} {
    global Test
    global WaitVar
    set elapsed [expr ([clock clicks] - $startTime) / 1000000.0]
    set fmt {%2d) %-35s : %5.1f sec. TITLE: %s}

    set rxp [regexp -nocase {title>([^<]*)</tit} [::http::data $token] all title]
    if {!$rxp} {
        set title "NO TITLE"
    }

    logData [format $fmt $id $url $elapsed $title]

    set of [open $Test(logFile)_$id w]
    puts $of [::http::data $token]
    close $of

    set WaitVar($id) 1
}

################################################################
# proc buildGUI {}--
#    Build the GUI for this display.
# Arguments
#   NONE
# 
# Results
#   A GUI magicly appears on your screen...
# 
proc buildGUI {} {
    text .t -yscrollcommand ".sb set"
    grid .t -row 1 -column 0 -columnspan 3 -sticky news
    grid rowconfigure . 1 -weight 1
    grid columnconfigure . 0 -weight 1
    
    scrollbar .sb -command ".t yview"
    grid .sb -row 1 -column 4 -sticky ns
    
    label .l -text "Test Running"
    grid .l -row 0 -column 0
    
    button .b -text "Quit" -command exit
    grid .b -row 0 -column 1
}

if {$argc < 2} {
    puts "$argv0 body.tcl statusLogFileIdentifier"
    exit
}

foreach {index} {body identifier} value $argv {
    set Test($index) $value
}

set Test(base) [file rootname $Test(body)]

set Test(statusLog) [open $Test(identifier)_[pid].log w]

buildGUI

source $Test(body)

close $Test(statusLog)

